<?php
/*
Plugin Name: Easy Attachments
Plugin URI: http://dannagle.com/easy-attachments/
Description: Easy Attachments gives the ability to append any number of files to your pages and posts. Includes settings to allow integrating with your theme.
Version: 1.0.0
Author: Dan Nagle
Author URI: http://dannagle.com/
*/

/*  Copyright 2010 Dan Nagle (email : daniel.a.nagle@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


// ===========
// = GLOBALS =
// ===========

global $wpdb;



// =========
// = HOOKS =
// =========
if( WP_ADMIN )
{
	add_action( 'admin_menu', 'attachments_init' );
	add_action( 'admin_head', 'attachments_init_js' );
	add_action( 'save_post',  'attachments_save' );
	add_action( 'admin_menu', 'attachments_menu' );
	add_action( 'admin_init', 'fix_async_upload_image' );
}
	add_filter( 'the_content', 'attachments_contentadder');

function attachments_format_bytes($size) {
    $units = array(' B', ' KiB', ' MiB', ' GiB', ' TiB');
    for ($i = 0; $size >= 1024 && $i < 4; $i++) $size /= 1024;
    return round($size, 1).$units[$i];
}


function attachments_contentadder ($content) {

global $post;

    $attachments = attachments_get_attachments($post->ID);
    $total_attachments = count($attachments);
    if(($total_attachments) > 0)
    {

	$attachment_showfilesize_check = get_option('attachment_showfilesize_check', true);
	$attachment_boldfilesize_check = get_option('attachment_boldfilesize_check', true);
	$attachment_showfilecaption_check = get_option('attachment_showfilecaption_check', true);
	$attachment_showlistingcount_check = get_option('attachment_showlistingcount_check', true);
	$attachment_rowcolor1 = get_option('attachment_rowcolor1', '#fdfcc9');
	$attachment_rowcolor2 = get_option('attachment_rowcolor2', '#faf99e');
	
	if(!is_singular())
	{
		if($attachment_showlistingcount_check == "on" || $attachment_showlistingcount_check == "true" || $attachment_showlistingcount_check == "checked")
		{
			return $content.'<br/>(Contains <a href="'.get_permalink($post->ID).'#attachments">'.$total_attachments.' attachments</a>.)'; 
		} else {
			return $content;
		}
	} else	{

$content .= '
<div id="attachments">
<p><h4>Attachments</h4><a name="attachments"></a></p>
';
      $i = 0;
      $content .= '<ol>';
      foreach($attachments as $attachment)
      {

	if(($i % 2) == 0)
	{
         $content .='<li style="background-color:'.$attachment_rowcolor1.';"><p>';

	} else {

         $content .='<li style="background-color:'.$attachment_rowcolor2.';"><p>';

	}
	$i++;
	

          $content .='<a href="'.$attachment['location'].'">'.$attachment['title'].'</a>';
	  
	  if($attachment_showfilesize_check == "on" || $attachment_showfilesize_check == "true" || $attachment_showfilesize_check == "checked")
	  {
		if($attachment_boldfilesize_check == "on" || $attachment_boldfilesize_check == "true" || $attachment_boldfilesize_check == "checked")
		{
			$content .= ' <b>('.$attachment['filesizeformatted'].')</b>';
		} else {
			$content .= ' <b>('.$attachment['filesizeformatted'].')</b>';
			
		}
	  }

	  if($attachment_showfilecaption_check == "on" || $attachment_showfilecaption_check == "true" || $attachment_showfilecaption_check == "checked")
	  {
		if(trim($attachment['caption']) != '')
		{
		      $content .='<br/>'.$attachment['caption'].'';
		}
	  }


        $content .='</p></li>';
      }
    $content .='</ol>';
 
$content .='</div>';
		
	}

    }

	return $content;
}


// =============
// = FUNCTIONS =
// =============

/**
 * Compares two array values with the same key "order"
 *
 * @param string $a First value
 * @param string $b Second value
 * @return int
 * @author Jonathan Christopher
 */
function attachments_cmp($a, $b)
{
	$a = intval( $a['order'] );
	$b = intval( $b['order'] );
	
	if( $a < $b )
	{
		return -1;
	}
	else if( $a > $b )
	{
		return 1;
	}
	else
	{
		return 0;
	}
}




/**
 * Creates the markup for the WordPress admin options page
 *
 * @return void
 * @author Jonathan Christopher
 */
function attachments_options()
{
	include 'easyattachments.options.php';
}




/**
 * Creates the entry for Attachments Options under Settings in the WordPress Admin
 *
 * @return void
 * @author Jonathan Christopher
 */
function attachments_menu()
{
	add_options_page('Settings', 'Easy Attachments', 8, __FILE__, 'attachments_options');
}




/**
 * Inserts HTML for meta box, including all existing attachments
 *
 * @return void
 * @author Jonathan Christopher
 */
function attachments_add()
{?>
	
	<div id="attachments-inner">
		
		<?php
			$media_upload_iframe_src = "media-upload.php?type=image&TB_iframe=1";
			$image_upload_iframe_src = apply_filters( 'image_upload_iframe_src', "$media_upload_iframe_src" );
		?>
		
		<ul id="attachments-actions">
			<li>
				<a id="attachments-thickbox" href="<?php echo $image_upload_iframe_src; ?>&attachments_thickbox=1" title="Attachments" class="button button-highlighted">
					Attach
				</a>
			</li>
		</ul>
		
		<div id="attachments-list">
			<input type="hidden" name="attachments_nonce" id="attachments_nonce" value="<?php echo wp_create_nonce( plugin_basename(__FILE__) ); ?>" />
			<ul>
				<?php
					if( !empty($_GET['post']) )
					{
						// get all attachments
						$existing_attachments = attachments_get_attachments( intval( $_GET['post'] ) );
						
						if( is_array($existing_attachments) && !empty($existing_attachments) )
						{
							$attachment_index = 0;
							foreach ($existing_attachments as $attachment) : $attachment_index++; ?>
								<li class="attachments-file">
									<h2>
										<a href="#" class="attachment-handle">
											<span class="attachment-handle-icon"><img src="<?php echo WP_PLUGIN_URL; ?>/attachments/images/handle.gif" alt="Drag" /></span>
										</a>
										<span class="attachment-name"><?php echo $attachment['name']; ?></span>
										<span class="attachment-delete"><a href="#"><?php _e("Delete", "attachments")?></a></span>
									</h2>
									<div class="attachments-fields">
										<div class="textfield" id="field_attachment_title_<?php echo $attachment_index ; ?>">
											<label for="attachment_title_<?php echo $attachment_index; ?>"><?php _e("Title", "attachments")?></label>
											<input type="text" id="attachment_title_<?php echo $attachment_index; ?>" name="attachment_title_<?php echo $attachment_index; ?>" value="<?php echo $attachment['title']; ?>" size="20" />
										</div>
										<div class="textfield" id="field_attachment_caption_<?php echo $attachment_index; ?>">
											<label for="attachment_caption_<?php echo $attachment_index; ?>"><?php _e("Caption", "attachments")?></label>
											<input type="text" id="attachment_caption_<?php echo $attachment_index; ?>" name="attachment_caption_<?php echo $attachment_index; ?>" value="<?php echo $attachment['caption']; ?>" size="20" />
										</div>
									</div>
									<div class="attachments-data">
										<input type="hidden" name="attachment_id_<?php echo $attachment_index; ?>" id="attachment_id_<?php echo $attachment_index; ?>" value="<?php echo $attachment['id']; ?>" />
										<input type="hidden" class="attachment_order" name="attachment_order_<?php echo $attachment_index; ?>" id="attachment_order_<?php echo $attachment_index; ?>" value="<?php echo $attachment['order']; ?>" />
									</div>
									<div class="attachment-thumbnail">
										<span class="attachments-thumbnail">
											<?php echo wp_get_attachment_image( $attachment['id'], array(80, 60), 1 ); ?>
										</span>
									</div>
								</li>
							<?php endforeach;
						}
					}
				?>
			</ul>
		</div>
		
	</div>
	
<?php }



/**
 * Creates meta box on all Posts and Pages
 *
 * @return void
 * @author Jonathan Christopher
 */

function attachments_meta_box()
{
	// for posts
	add_meta_box( 'attachments_list', __( 'Easy Attachments', 'attachments_textdomain' ), 'attachments_add', 'post', 'normal' );
	
	// for pages
	add_meta_box( 'attachments_list', __( 'Easy Attachments', 'attachments_textdomain' ), 'attachments_add', 'page', 'normal' );
	
	// for custom post types
	if( function_exists( 'get_post_types' ) )
	{
		$args = array(
			'public'   => true,
			'_builtin' => false
			); 
		$output = 'objects';
		$operator = 'and';
		$post_types = get_post_types( $args, $output, $operator );
		foreach($post_types as $post_type)
		{
			if (get_option('attachments_cpt_' . $post_type->name)=='true')
			{
				add_meta_box( 'attachments_list', __( 'Easy Attachments', 'attachments_textdomain' ), 'attachments_add', $post_type->name, 'normal' );
			}
		}
	}
}



/**
 * Echos JavaScript that sets some required global variables
 *
 * @return void
 * @author Jonathan Christopher
 */
function attachments_init_js()
{
	global $pagenow;
	
	echo '<script type="text/javascript" charset="utf-8">';
	echo '	var attachments_base = "' . WP_PLUGIN_URL . '/attachments"; ';
	echo '	var attachments_media = ""; ';
	if ( 'media-upload.php' == $pagenow || 'async-upload.php' == $pagenow )
	{
		echo '	var attachments_upload = true; ';
	}
	else
	{
		echo '	var attachments_upload = false; ';
	}
	if( ( 'media-upload.php' == $pagenow || 'async-upload.php' == $pagenow ) && is_attachments_context() )
	{
		echo '	var attachments_is_attachments_context = true; ';
	}
	else
	{
		echo '	var attachments_is_attachments_context = false; ';
	}
	echo '</script>';
}



/**
 * Fired when Post or Page is saved. Serializes all attachment data and saves to post_meta
 *
 * @param int $post_id The ID of the current post
 * @return void
 * @author Jonathan Christopher
 * @author JR Tashjian
 */
function attachments_save($post_id)
{
	// verify this came from the our screen and with proper authorization,
	// because save_post can be triggered at other times
	if ( !wp_verify_nonce( $_POST['attachments_nonce'], plugin_basename(__FILE__) )) {
		return $post_id;
	}

	// verify if this is an auto save routine. If it is our form has not been submitted, so we dont want
	// to do anything
	if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) 
		return $post_id;

	// Check permissions
	if ( 'page' == $_POST['post_type'] ) {
		if ( !current_user_can( 'edit_page', $post_id ) )
			return $post_id;
	} else {
		if ( !current_user_can( 'edit_post', $post_id ) )
			return $post_id;
	}

	// OK, we're authenticated: we need to find and save the data
	
	// delete all current attachments meta
	// moved outside conditional, else we can never delete all attachments
	delete_post_meta($post_id, '_attachments');
	
	// Since we're allowing Attachments to be sortable, we can't simply increment a counter
	// we need to keep track of the IDs we're given
	$attachment_ids = array();
	
	// We'll build our array of attachments
	foreach($_POST as $key => $data) {
		
		// Arbitrarily using the id
		if( substr($key, 0, 14) == 'attachment_id_' )
		{
			array_push( $attachment_ids, substr( $key, 14, strlen( $key ) ) );
		}
		
	}
		
	// If we have attachments, there's work to do
	if( !empty( $attachment_ids ) )
	{
		
		foreach ( $attachment_ids as $i )
		{
			if( !empty( $_POST['attachment_id_' . $i] ) )
			{
				$attachment_details = array(
						'id' 				=> $_POST['attachment_id_' . $i],
						'title' 			=> str_replace( '"', '&quot;', $_POST['attachment_title_' . $i] ),
						'caption' 			=> str_replace( '"', '&quot;', $_POST['attachment_caption_' . $i] ),
						'order' 			=> $_POST['attachment_order_' . $i]
					);
				
				// serialize data and encode
				$attachment_serialized = base64_encode( serialize( $attachment_details ) );
				
				// add individual attachment
				add_post_meta( $post_id, '_attachments', $attachment_serialized );
			}
		}
		
	}
	
}



/**
 * Retrieves all Attachments for provided Post or Page
 *
 * @param int $post_id (optional) ID of target Post or Page, otherwise pulls from global $post
 * @return array $post_attachments
 * @author Jonathan Christopher
 * @author JR Tashjian
 */
function attachments_get_attachments( $post_id=null )
{
	global $post;
	
	if( $post_id==null )
	{
		$post_id = $post->ID;
	}
	
	// get all attachments
	$existing_attachments = get_post_meta( $post_id, '_attachments', false );
	
	if( !empty( $existing_attachments ) )
	{
	    $legacy_existing_attachments = unserialize( $existing_attachments[0] );
	}
	
	// Check for legacy attachments
	if( is_array( $legacy_existing_attachments ) )
	{
		$tmp_legacy_attachments = array();
		
		// Legacy attachments (single serialized record)
		foreach ( $legacy_existing_attachments as $legacy_attachment )
		{
			array_push( $tmp_legacy_attachments, base64_encode( serialize( $legacy_attachment ) ) );
		}
		
		$existing_attachments = $tmp_legacy_attachments;
	}
	
	// We can now proceed as normal, all legacy data should now be upgraded
	if( is_array( $existing_attachments ) && count( $existing_attachments ) > 0 )
	{
		$post_attachments = array();
		
		foreach ($existing_attachments as $attachment)
		{
			// decode and unserialize the data
			$data = unserialize( base64_decode( $attachment ) );
			
			array_push( $post_attachments, array(
				'id' 			=> stripslashes( $data['id'] ),
				'name' 			=> stripslashes( get_the_title( $data['id'] ) ),
				'mime' 			=> stripslashes( get_post_mime_type( $data['id'] ) ),
				'title' 		=> stripslashes( $data['title'] ),
				'caption' 		=> stripslashes( $data['caption'] ),
				'location' 		=> stripslashes( wp_get_attachment_url( $data['id'] ) ),
				'order' 		=> stripslashes( $data['order'] ),
				'link'			=> stripslashes( wp_get_attachment_link( $data['id'] ) ),
				'path'			=> get_attached_file($data['id']),
				'filesize'		=> filesize(get_attached_file($data['id'])),
				'filesizeformatted'	=> attachments_format_bytes(filesize(get_attached_file($data['id'])))
			));
		}
		
		// sort attachments
		if( count( $post_attachments ) > 1 )
		{
			usort( $post_attachments, "attachments_cmp" );
		}
	}
	
	return $post_attachments;
}


if( !function_exists( 'fix_async_upload_image' ) )
{
	function fix_async_upload_image() {
		if( isset( $_REQUEST['attachment_id'] ) )
		{
			$GLOBALS['post'] = get_post( $_REQUEST['attachment_id'] );
		}
	}
}

function is_attachments_context()
{
	global $pagenow;
	
	// if post_id is set, it's the editor upload...
	if ( ( 'media-upload.php' == $pagenow || 'async-upload.php' == $pagenow ) && empty( $_REQUEST['post_id'] ) )
	{
		return true;
	}
	return false;
}

function hijack_thickbox_text($translated_text, $source_text, $domain)
{
	if ( is_attachments_context() )
	{
		if ('Insert into Post' == $source_text) {
			return __('Attach', 'attachments' );
		}
	}
	return $translated_text;
}





/**
 * This is the main initialization function, it will invoke the necessary meta_box
 *
 * @return void
 * @author Jonathan Christopher
 */

function attachments_init()
{
	global $pagenow;
	
	wp_enqueue_script( 'jquery-ui-core' );
	wp_enqueue_style( 'thickbox' );
	
	if ( 'media-upload.php' == $pagenow || 'async-upload.php' == $pagenow )
	{
		add_filter( 'gettext', 'hijack_thickbox_text', 1, 3 );
	}

	wp_enqueue_style( 'easyattachments', WP_PLUGIN_URL . '/easy-attachments/css/easyattachments.css' );
	wp_enqueue_style( 'farbtastic', WP_PLUGIN_URL . '/easy-attachments/css/farbtastic.css' );
	wp_enqueue_script( 'easyattachments', WP_PLUGIN_URL . '/easy-attachments/js/easyattachments.js', array( 'thickbox' ), false, false );
	wp_enqueue_script( 'farbtastic', WP_PLUGIN_URL . '/easy-attachments/js/farbtastic.js');

	if( function_exists( 'load_plugin_textdomain' ) )
	{
		if( !defined('WP_PLUGIN_DIR') )
		{
			load_plugin_textdomain( 'easyattachments', str_replace( ABSPATH, '', dirname( __FILE__ ) ) );
		}
		else
		{
			load_plugin_textdomain( 'easyattachments', false, dirname( plugin_basename( __FILE__ ) ) );
		}
	}

	attachments_meta_box();
}